# -*- coding: utf-8 -*-

"""Unit tests for utils module."""
import os
import filecmp
import unittest
import pandas
import etlstat.text.utils as utils


class TestUtils(unittest.TestCase):
    """Testing methods for util functions."""

    def setUp(self):
        """Initialize the test variables and configuration."""
        self.current_dir = os.path.dirname(os.path.abspath(__file__))

    def test_parsed_columns(self):
        """Check column parsing has the desired effect."""
        df = pandas.DataFrame
        df.columns = ['Período',
                      'Clasificación CNAE 2009',
                      'Número de trabajadores',
                      'Índice de variación']
        utils.parse_df_columns(df)
        self.assertEqual(df.columns[0], 'periodo')
        self.assertEqual(df.columns[1], 'clasificacion_cnae_2009')
        self.assertEqual(df.columns[2], 'numero_de_trabajadores')
        self.assertEqual(df.columns[3], 'indice_de_variacion')

    def test_bulk_replace_url_in_xml(self):
        """Check a URL replacement takes place in all files in a directory."""
        utils.bulk_replace_url_in_xml(f'''{self.current_dir}/data/''')
        self.assertTrue(filecmp.cmp(f'''{self.current_dir}'''
                                    f'''/data/jobs/TODAS_Uso de Comercio '''
                                    f'''Electrónico.kjb''',
                                    f'''{self.current_dir}'''
                                    f'''/data/jobs/TODAS_Uso de Comercio '''
                                    f'''Electrónico.kjb'''))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""Integration tests for oracle database module."""

import os
import unittest

from etlstat.database.oracle import Oracle

import pandas

from sqlalchemy.exc import DatabaseError

os.environ['NLS_LANG'] = '.AL32UTF8'


class TestSqlLoader(unittest.TestCase):
    """Testing methods for Oracle class."""

    user = 'test'
    password = 'password'
    host = 'localhost'
    port = '1521'
    service_name = 'xe'
    conn_params = [user, password, host, port, service_name]
    output_path = os.getcwd() + '/etlstat/database/test/'
    os_path = '/usr/local/bin:/usr/bin:/bin:' \
        '/opt/oracle/instantclient_21_4'
    os_ld_library_path = '/opt/oracle/instantclient_21_4'

    @classmethod
    def setUpClass(cls):
        """Set up test variables."""
        user = 'system'
        password = 'oracle'
        host = 'localhost'
        port = '1521'
        service_name = 'xe'
        conn_params = [user, password, host, port, service_name]
        ora_conn = Oracle(*conn_params)
        try:
            sql = f"""CREATE USER test IDENTIFIED BY password
                DEFAULT TABLESPACE USERS TEMPORARY TABLESPACE TEMP;"""
            ora_conn.execute(sql)
        except DatabaseError:
            pass
        sql = f"""ALTER USER test QUOTA UNLIMITED ON USERS;
            GRANT CONNECT, RESOURCE TO test;"""
        ora_conn.execute(sql)

    def test_insert(self):
        """Check if a bulk insert with sql loader is correctly executed."""
        ora_conn = Oracle(*self.conn_params)
        sql = """CREATE TABLE px_01001 (id INTEGER, tipo_indicador
            VARCHAR(100), nivel_educativo VARCHAR(100), valor NUMBER)"""
        try:
            ora_conn.execute(sql)
        except DatabaseError:
            pass
        px_01001 = ora_conn.get_table('px_01001')
        self.assertTrue(px_01001.exists)
        source_file = self.output_path + 'px_01001.csv'
        data = pandas.read_csv(
            source_file,
            sep=';',
            encoding='utf8')
        data.name = 'px_01001'
        ora_conn.insert(
            *self.conn_params,
            data_table=data,
            output_path=self.output_path,
            os_path=self.os_path,
            os_ld_library_path=self.os_ld_library_path,
            mode='TRUNCATE',
            columns=['*'],
            schema=self.conn_params[0]
        )

        sql = "SELECT COUNT(id) AS n FROM px_01001"
        try:
            result = ora_conn.execute(sql)
            assert result[0]['n'][0] == 21
        except DatabaseError as dbe:
            print(str(dbe))


if __name__ == '__main__':
    unittest.main()



"""
This module manages PostgreSQL primitives.

    Date:
        25/09/2019

    Author:
        lla11358

    Version:
        0.1

    Notes:

"""

import logging
import sqlparse
from sqlalchemy import create_engine, text
from sqlalchemy.exc import DatabaseError, InterfaceError, DataError,\
    OperationalError,IntegrityError,InternalError,ProgrammingError,\
    NotSupportedError
import pandas as pd

logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger(__name__)


class PostgreSQL:
    """
    Manage connections to Postgresql databases.

    Postgresql class offers some helper methods that encapsulate
    primitive logic to interactuate with the database: insert/upsert,
    execute, drop, etc.
    Some primitives are not included because they can be handled more
    properly with sqlalchemy.

    """

    def __init__(self, *conn_params):
        """
        Initialize the database connection and other relevant data.

        Args:
            *conn_params: list with the following connection parameters:
                user(string): database user to connect to the schema.
                password(string): database password of the user.
                host(string): database management system host.
                port(string): tcp port where the database is listening.
                database(string): schema or database name.

        """
        # connection string in sqlalchemy format
        self.conn_string = f'''postgresql://{conn_params[0]}:''' + \
            f'''{conn_params[1]}@{conn_params[2]}:''' + \
            f'''{str(conn_params[3])}/{conn_params[4]}'''
        self.engine = create_engine(self.conn_string)
        self.database = conn_params[4]

    def execute(self, sql, **kwargs):
        """
        Execute a DDL or DML SQL statement.

            Args:
                sql (string): SQL statement(s) separated by semicolons (;)
                kwargs (dict): optional statement named parameters
            Returns:
                results (list): list of dataframes. Non-SELECT statements
                    returns empty dataframes.

        """
        results = []
        statements = sqlparse.split(sql)
        connection = self.engine.connect()
        # begin transaction
        trans = connection.begin()
        try:
            for statement in statements:
                result_set = pd.DataFrame()
                result = connection.execute(
                    text(statement.strip(';')), **kwargs)
                if result.returns_rows:
                    result_set = pd.DataFrame(result.fetchall())
                    result_set.columns = result.keys()
                    LOGGER.info('Number of returned rows: %s',
                                str(len(result_set.index)))
                results.append(result_set)
            # end transaction
            trans.commit()
        except InterfaceError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except DatabaseError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except DataError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except OperationalError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise        
        except IntegrityError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except InternalError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except ProgrammingError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except NotSupportedError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        finally:
            connection.close()
        return results



# coding: utf-8
"""
This module manages MySQL primitives.

    Date:
        27/11/2018

    Author:
        emm13775

    Version:
        0.1

    Notes:

"""

import os
import logging
import sqlparse
from sqlalchemy import create_engine, text, select, func, MetaData, Table
from sqlalchemy.exc import DatabaseError, InterfaceError, DataError,\
    OperationalError,IntegrityError,InternalError,ProgrammingError,\
    NotSupportedError
import pandas as pd

logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger(__name__)


class MySQL:
    """
    Manage connections to MySQL databases.

    MySQL class offers some helper methods that encapsulate primitive logic
    to interactuate with the database: insert/upsert, execute, drop, etc.
    Some primitives are not included because they can be handled more
    properly with sqlalchemy.

    """

    def __init__(self, *conn_params):
        """
        Initialize the database connection and other relevant data.

        Args:
            *conn_params: list with the following connection parameters:
                user(string): database user to connect to the schema.
                password(string): database password of the user.
                host(string): database management system host.
                port(string): tcp port where the database is listening.
                database(string): schema or database name.

        """
        # connection string in sqlalchemy format
        self.conn_string = f'''mysql+mysqlconnector://{conn_params[0]}:''' + \
            f'''{conn_params[1]}@{conn_params[2]}:''' + \
            f'''{str(conn_params[3])}/{conn_params[4]}'''
        self.engine = create_engine(self.conn_string)
        self.database = conn_params[4]

    def get_table(self, table_name, schema=None):
        """
        Get a database table into a sqlalchemy Table object.

        Args:
            table_name(string): name of the database table to map.
            schema(string): name of the schema to which the table belongs.
                        Defaults to the selected database in the
                        connection.
        Returns:
            table(Table): sqlalchemy Table object referencing the specified
                            database table.

        """
        if not schema:
            schema = self.database
        meta = MetaData(bind=self.engine, schema=schema)
        return Table(table_name, meta, autoload=True,
                     autoload_with=self.engine)

    def execute(self, sql, **kwargs):
        """
        Execute a DDL or DML SQL statement.

            Args:
                sql (string): SQL statement(s) separated by semicolons (;)
                kwargs (dict): optional statement named parameters
            Returns:
                results (list): list of dataframes. Non-SELECT statements
                    returns empty dataframes.

        """
        results = []
        statements = sqlparse.split(sql)
        connection = self.engine.connect()
        # begin transaction
        trans = connection.begin()
        try:
            for statement in statements:
                result_set = pd.DataFrame()
                result = connection.execute(
                    text(statement.strip(';')), **kwargs)
                if result.returns_rows:
                    result_set = pd.DataFrame(result.fetchall())
                    result_set.columns = result.keys()
                    LOGGER.info('Number of returned rows: %s',
                                str(len(result_set.index)))
                results.append(result_set)
            # end transaction
            trans.commit()
        except InterfaceError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except DatabaseError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except DataError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except OperationalError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise        
        except IntegrityError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except InternalError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except ProgrammingError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except NotSupportedError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        finally:
            connection.close()
        return results

    def drop(self, table_name, schema=None):
        """
        Drop a table from the database.

        Args:
          table_name(str): name of the table to drop.
          schema (string): optional database name.

        Returns: nothing.

        """
        if not schema:
            schema = self.database
        db_table = self.get_table(table_name, schema)
        db_table.drop(self.engine, checkfirst=True)
        LOGGER.info('Table %s.%s successfully dropped.', schema, table_name)

        # Placeholders can only represent VALUES. You cannot use them for
        # sql keywords/identifiers.

    def insert(self, data_table, if_exists='fail',
               columns=['*'], schema=None, rm_tmp=True):
        r"""
        Insert a dataframe into a table.

        Converts the dataframe to CSV format and bulk loads it.

        Args:
          data_table(Dataframe): dataframe with the data to load.
                                 Must contain the target table name in
                                 its name attribute.
          if_exists(string): {‘fail’, ‘replace’, ‘append’}, default ‘fail’.
                             See `pandas.to_sql()` for details. Warning: if
                             replace is chosen, PKs will be deleted and will
                             have to be recreated.
          columns(list): list of str containing the column names to load to a
                         table. Defaults to ['*'] (all columns).
          schema (string): name of the database that contains the
                           destination table.
          rm_tmp (bool): remove or not the temporary csv file. Defaults to
                         True.
        Returns:
          db_table(Table): sqlalchemy table mapping the table with the inserted
                           records.

        """
        tmpfile = 'tmp.csv'  # filename for temporary file to load from
        sep = ';'  # separator for temp file
        quotechar = '"'  # Character used to quote fields
        line_terminated_by = '\n'  # termination character for file lines

        if columns == ['*']:
            columns = ', '.join(data_table.columns)
        else:
            columns = ', '.join(columns)

        if not schema:
            schema = self.database

        connection = self.engine.connect()
        db_table = Table()
        if isinstance(data_table, pd.DataFrame):
            data_table.to_sql(data_table.name, self.engine,
                                  if_exists=if_exists, index=False)
        else:
            raise TypeError("data_table must be a DataFrame.")
        try:
            LOGGER.info('creating %s ok', tmpfile)
            data_table.to_csv(tmpfile, na_rep='\\N', header=False,
                              index=False, sep=sep, quotechar=quotechar)
            LOGGER.info('loading %s ok', tmpfile)
            sql_load = f'''LOAD DATA LOCAL INFILE '{tmpfile}' INTO TABLE
                           {schema}.{data_table.name}
                           FIELDS TERMINATED BY '{sep}'
                           OPTIONALLY ENCLOSED BY '{quotechar}'
                           LINES TERMINATED BY '{line_terminated_by}'
                           ({columns});'''
            connection.execute(sql_load)
            if rm_tmp:
                os.remove(tmpfile)
            db_table = self.get_table(data_table.name)
            row_count = connection.engine.scalar(
                select([func.count('*')]).select_from(db_table))
            LOGGER.info('Number of inserted rows: %s', str(row_count))
        except Exception as exception:
            LOGGER.error(exception)
            raise
        finally:
            connection.close()
        return db_table

    def upsert(self, tmp_data, table_name, sql, if_exists='fail',
               columns=['*'], rm_tmp=True, schema=None):
        r"""
        Update/insert a dataframe into a table.

        Converts the dataframe to CSV format, and bulk loads it to a temporary
        table and then executing a raw update or update/insert query from a
        text file which extracts records from the temporary table and loads
        them into the definitive one.

        Args:
            tmp_data(Dataframe): dataframe with the data to load in a
                                    temporary table.
            table_name(String): name of the table to be
                                updated/inserted to.
            sql(string): string with the SQL update/insert query.
            if_exists(string): {‘fail’, ‘replace’, ‘append’}, default ‘fail’.
                               See `pandas.to_sql()` for details. Warning: if
                               replace is chosen, PKs will be deleted and will
                               have to be recreated.
            columns(list): list of str containing the column names to load to a
                         table. Defaults to ['*'] (all columns).
            rm_tmp(Boolean): Defauls to True. Determines if the temporary
                             table should be dropped (expected behaviour)
                             or not (for debugging purposes). It affects to
                             csv temporary file created by 'insert' method.
            schema (string): name of the database that contains the
                             destination table.
        Returns:
            db_table(Table): sqlalchemy table mapping the table with the
                                inserted/updated records.

        """
        if not schema:
            schema = self.database

        connection = self.engine.connect()
        try:
            self.insert(tmp_data, if_exists=if_exists, columns=columns,
                        rm_tmp=rm_tmp, schema=schema)
            connection.execute(sql)  # update/insert query
            if rm_tmp:
                self.drop(tmp_data.name)  # remove temporary table
            db_table = self.get_table(table_name)
            row_count = connection.engine.scalar(
                select([func.count('*')]).select_from(db_table)
            )
            LOGGER.info('Number of rows in the updated table: %s',
                        str(row_count))
        except Exception as exception:
            LOGGER.exception(exception)
            raise
        finally:
            connection.close()
        return db_table



# coding: utf-8
"""
This module manages Oracle primitives.

    Date:
        23/01/2018

    Author:
        lla11358

    Version:
        0.1
"""

import csv
import logging
import os
import shlex
import subprocess

import pandas as pd

from sqlalchemy import MetaData, Table, create_engine, text
from sqlalchemy.exc import DatabaseError, InterfaceError, DataError,\
    OperationalError,IntegrityError,InternalError,ProgrammingError,\
    NotSupportedError

import sqlparse

logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger(__name__)


class Oracle:
    """
    Manage connections to Oracle databases.

        Oracle class offers some helper methods that encapsulate primitive
        logic to interactuate with the database: insert/upsert, execute,
        drop, etc.
        Some primitives are not included because they can be handled more
        properly with sqlalchemy.

    """

    def __init__(self, *conn_params, encoding='utf8'):
        """
        Initialize the database connection and other relevant data.

            Args:
                *conn_params: list of the following connection parameters:
                    user(string): database user to connect to the schema.
                    password(string): database password of the user.
                    host(string): database management system host.
                    port(string): tcp port where the database is listening.
                    service_name(string): Oracle instance name.
                encoding (string): Charset encoding.

        """
        # connection string in sqlalchemy format
        self.conn_string = f"""oracle+cx_oracle://{conn_params[0]}:""" + \
            f"""{conn_params[1]}@{conn_params[2]}:""" + \
            f"""{conn_params[3]}/?service_name={conn_params[4]}"""
        self.engine = create_engine(self.conn_string,
                                    encoding=encoding,
                                    coerce_to_unicode=True,
                                    coerce_to_decimal=False)
        self.schema = conn_params[0]
        self.encoding = encoding

    def get_table(self, table_name, schema=None):
        """
        Get a database table into a sqlalchemy Table object.

            Args:
                table_name(string): name of the database table to map.
                schema(string): name of the schema to which the table belongs.
                    Defaults to the selected database in the connection.
            Returns:
                table(Table): sqlalchemy Table object referencing the specified
                    database table.

        """
        if not schema:
            schema = self.schema

        meta = MetaData(bind=self.engine, schema=schema)
        return Table(table_name, meta, autoload=True,
                     autoload_with=self.engine)

    def execute(self, sql, **kwargs):
        """
        Execute a DDL or DML SQL statement.

            Args:
                sql (string): SQL statement(s) separated by semicolons (;)
                kwargs (dict): optional statement named parameters
            Returns:
                results (list): list of dataframes. Non-SELECT statements
                    returns empty dataframes.

        """
        results = []
        statements = sqlparse.split(sql)
        connection = self.engine.connect()
        # begin transaction
        trans = connection.begin()
        try:
            for statement in statements:
                result_set = pd.DataFrame()
                result = connection.execute(
                    text(statement.strip(';')), **kwargs)
                if result.returns_rows:
                    result_set = pd.DataFrame(result.fetchall())
                    result_set.columns = result.keys()
                    LOGGER.info('Number of returned rows: %s',
                                str(len(result_set.index)))
                results.append(result_set)
            # end transaction
            trans.commit()
        except InterfaceError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except DatabaseError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except DataError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except OperationalError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise        
        except IntegrityError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except InternalError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except ProgrammingError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        except NotSupportedError as db_error:
            trans.rollback()
            LOGGER.error(db_error)
            raise
        finally:
            connection.close()
        return results

    def drop(self, table_name, schema=None):
        """
        Drop a table from the database.

        Args:
          table_name(string): name of the table to drop.
          schema (string): optional schema name.

        Returns: nothing.

        """
        if not schema:
            schema = self.schema

        db_table = self.get_table(table_name, schema=schema)
        db_table.drop(self.engine, checkfirst=True)
        LOGGER.info('Table %s.%s successfully dropped.', schema, table_name)

        # Placeholders can only represent VALUES. You cannot use them for
        # sql keywords/identifiers.

    @staticmethod
    def insert(
            *conn_params,
            data_table,
            output_path,
            os_path,
            os_ld_library_path,
            mode='INSERT',
            columns=['*'],
            schema=None,
            errors='0',
            remove_data=True):
        """
        Insert a dataframe into a table via Oracle SQL Loader.

        Extracts field names and data values from a Pandas DataFrame.
        Requires Oracle Instant Client and Tools installed in the workstation.
        Destination table must exist in the database.
        Usage of SQL Loader:

            sqlldr <user>/<password> control=<control_file> [log=<log_file>]
            [bad=bad_file]

        Args:
            *conn_params: list of the following connection parameters:
                user(string): database user to connect to the schema.
                password(string): database password of the user.
                host(string): database management system host.
                port(string): tcp port where the database is listening.
                service_name(string): Oracle instance name.
            data_table(Dataframe): dataframe with the data to load. Must
                contain the target table name in its 'name' attribute.
            output_path (str): path for output data files
            os_path (str): PATH environment variable
            os_ld_library_path (str): LD_LIBRARY_PATH environment variable
            mode (str): insertion mode:
                INSERT: Specifies that you are loading into an empty table.
                    SQL*Loader will abort the load if the table contains data
                    to start with. This is the default.
                APPEND: If we want to load the data into a table which is
                    already containing some rows.
                REPLACE: Specifies that, we want to replace the data in the
                    table before loading. Will 'DELETE' all the existing
                    records and replace them with new.
                TRUNCATE: This is same as 'REPLACE', but SQL*Loader will
                    use the 'TRUNCATE' command instead of 'DELETE' command.
            columns (list): list of str containing the column names to load to
                a table. Defaults to ['*'] (all columns).
            schema (str): database schema. Defaults to connection 'user'.
            errors (str): number of errors to allow. Defaults to 0 (no errors
                allowed).
            remove_data (bool): to remove or not the log and data files
                generated. Defaults to True.

        """
        if columns == ['*']:
            columns = ', '.join(data_table.columns)
        else:
            columns = ', '.join(columns)

        if not schema:
            schema = conn_params[0]

        # control file
        ctl_file = open(f"""{output_path}{data_table.name}.ctl""",
                        mode='w',
                        encoding='utf8')
        ctl_header = f"""LOAD DATA
                     CHARACTERSET UTF8
                     INFILE '{output_path}{data_table.name}.dat'
                     {mode}
                     INTO TABLE {schema}.{data_table.name}
                     FIELDS TERMINATED BY ';' OPTIONALLY ENCLOSED BY '\"'
                     TRAILING NULLCOLS
                     ({columns})"""

        ctl_file.write(ctl_header)
        ctl_file.close()

        # data file
        data_table.to_csv(
            f"""{output_path}{data_table.name}.dat""",
            columns=data_table.columns,
            sep=';',
            header=False,
            index=False,
            doublequote=True,
            quoting=csv.QUOTE_NONNUMERIC,
            encoding='utf8')

        # set environment variables
        env = os.environ.copy()
        env['PATH'] = os_path
        env['LD_LIBRARY_PATH'] = os_ld_library_path

        # generate SQL Loader arguments
        os_command = f"""sqlldr {conn_params[0]}/{conn_params[1]}""" + \
            f"""@{conn_params[2]}:{conn_params[3]}/{conn_params[4]} """ + \
            f"""control='{output_path}{data_table.name}.ctl' """ + \
            f"""log='{output_path}{data_table.name}.log' """ + \
            f"""bad='{output_path}{data_table.name}.bad' """ + \
            f"""errors={errors}"""
        args = shlex.split(os_command)

        # execution of Oracle SQL Loader
        try:
            subprocess.call(args, env=env)
        except subprocess.SubprocessError as sproc_error:
            LOGGER.error(sproc_error)

        # deleting output files
        if remove_data:
            try:
                os.remove(f"""{output_path}{data_table.name}.dat""")
                os.remove(f"""{output_path}{data_table.name}.log""")
                os.remove(f"""{output_path}{data_table.name}.bad""")
            except FileNotFoundError:
                pass



# -*- coding: utf-8 -*-

"""Integration tests for oracle database module."""

import os
import unittest

from etlstat.database.oracle import Oracle

import pandas

from sqlalchemy import (Boolean, Column, DateTime, Float, Integer, String,
                        func, select)
from sqlalchemy.exc import DatabaseError, InvalidRequestError
from sqlalchemy.ext.declarative import declarative_base


os.environ['NLS_LANG'] = '.AL32UTF8'


class TestOracle(unittest.TestCase):
    """Testing methods for Oracle class."""

    user = 'test'
    password = 'password'
    host = 'localhost'
    port = '1521'
    service_name = 'xe'
    conn_params = [user, password, host, port, service_name]
    output_path = os.getcwd() + '/etlstat/database/test/'

    @classmethod
    def setUpClass(cls):
        """Set up test variables."""
        user = 'system'
        password = 'oracle'
        host = 'localhost'
        port = '1521'
        service_name = 'xe'
        conn_params = [user, password, host, port, service_name]
        ora_conn = Oracle(*conn_params)
        try:
            sql = """CREATE USER test IDENTIFIED BY password
                  DEFAULT TABLESPACE USERS TEMPORARY TABLESPACE TEMP"""
            ora_conn.execute(sql)
        except DatabaseError:
            pass
        sql = """ALTER USER test QUOTA UNLIMITED ON USERS;
                 GRANT CONNECT, RESOURCE TO test"""
        ora_conn.execute(sql)

    def test_init(self):
        """Check connection with Oracle database."""
        self.assertEqual(
            str(Oracle(*self.conn_params).engine),
            "Engine(oracle+cx_oracle://test:***@localhost:1521/?service_name=xe)")

    def test_execute(self):
        """Check if different queries are correctly executed."""
        ora_conn = Oracle(*self.conn_params)
        sql = """CREATE TABLE table1 (id integer, column1 varchar2(100),
              column2 number)"""
        ora_conn.execute(sql)
        table1 = ora_conn.get_table('table1')
        self.assertEqual(table1.c.column1.name, 'column1')
        sql = """INSERT INTO table1 (id, column1, column2)
              VALUES (1, 'Varchar text (100 char)',
              123456789.012787648484859)"""
        ora_conn.execute(sql)  # EXECUTE example
        # The select.columns parameter is not available in the method form of
        # select(), e.g. FromClause.select().
        # See https://docs.sqlalchemy.org/en/latest/core/selectable.html#
        # sqlalchemy.sql.expression.FromClause.select
        results = ora_conn.engine.execute(
            select([table1.c.column1]).select_from(table1)).fetchall()
        expected = 'Varchar text (100 char)'
        current = results[0][0]
        # this returns a tuple inside a list and I dont know why
        self.assertEqual(expected, current)
        query = f"""select * from table1 order by id"""
        result = ora_conn.execute(query)
        expected = 1
        current = len(result[0].index)
        self.assertEqual(expected, current)
        ora_conn.drop('table1')

    def test_execute_multiple(self):
        """Check if multiple SQL statements are correctly executed."""
        ora_conn = Oracle(*self.conn_params)
        sql = """CREATE TABLE table1 (id integer, column1 varchar2(100),
              column2 number);
              INSERT INTO table1 (id, column1, column2)
              VALUES (1, 'Varchar; text; (100 char)',
              123456789.012787648484859);
              INSERT INTO table1 (id, column1, column2)
              VALUES (2, 'Varchar; text; (100 char)',
              -789.0127876);
              SELECT id, column2 FROM table1;"""
        results = ora_conn.execute(sql)
        self.assertEqual(len(results), 4)
        self.assertEqual(len(results[3].index), 2)
        ora_conn.drop('table1')

    def test_select(self):
        """Check select statement using sqlalchemy."""
        ora_conn = Oracle(*self.conn_params)
        table_name = "audit_actions"
        schema = "sys"
        audit_actions = ora_conn.get_table(table_name, schema=schema)
        results = ora_conn.engine.execute(
            select('*').where(
                audit_actions.c.name.like('CREATE%')).where(
                    audit_actions.c.action > 100).select_from(
                        audit_actions)).fetchall()
        table_df = pandas.DataFrame(results)
        self.assertGreaterEqual(len(table_df), 17)

    def test_get_table(self):
        """Check get table from the database using SqlAlchemy."""
        ora_conn = Oracle(*self.conn_params)
        hlp = ora_conn.get_table('help', schema='system')  # GET TABLE example
        row_count = ora_conn.engine.scalar(
            select([func.count('*')]).select_from(hlp)
        )
        # The select.columns parameter is not available in the method form of
        # select(), e.g. FromClause.select().
        # See https://docs.sqlalchemy.org/en/latest/core/selectable.html#
        # sqlalchemy.sql.expression.FromClause.select
        ora_conn.engine.execute(
            select([hlp.c.info]).select_from(hlp))
        self.assertEqual(row_count, 919)

    def test_create(self):
        """Check create table using sqlalchemy."""
        Base = declarative_base()
        ora_conn = Oracle(*self.conn_params)

        # table creation can be done via execute() + raw SQL or using this:
        class Table2(Base):
            """Auxiliary sqlalchemy table model for the tests."""

            __tablename__ = 'table2'

            column_int = Column(Integer)
            column_string = Column(String(20))
            column_float = Column(Float)
            column_datetime = Column(DateTime)
            column_boolean = Column(Boolean)
            id = Column(Integer, primary_key=True)

        Table2.__table__.create(bind=ora_conn.engine)
        table2 = ora_conn.get_table('table2')
        self.assertEqual(table2.c.column_datetime.name, 'column_datetime')
        self.assertEqual(len(table2.c), 6)
        ora_conn.drop('table2')

    def test_drop(self):
        """Check drop for an existing table."""
        ora_conn = Oracle(*self.conn_params)
        sql = "CREATE TABLE table1 (id INTEGER, column1 VARCHAR(100), " \
            "column2 NUMBER)"
        ora_conn.execute(sql)
        ora_conn.get_table('table1')
        ora_conn.drop('table1')  # DROP example
        with self.assertRaises(InvalidRequestError):
            ora_conn.get_table('table1')

    def test_delete(self):
        """Check delete rows from table."""
        ora_conn = Oracle(*self.conn_params)
        sql = f"""CREATE TABLE test_delete (column_int INTEGER,
            column_string VARCHAR(100), column_float NUMBER);
            INSERT INTO test_delete (column_int, column_string,
            column_float) VALUES(2, 'string2', 456.956);
            INSERT INTO test_delete (column_int, column_string,
            column_float) VALUES(1, 'string1', 38.905);"""
        ora_conn.execute(sql)
        table = ora_conn.get_table('test_delete')
        expected = 2
        current = ora_conn.engine.scalar(
            select([func.count('*')]).select_from(table)
        )
        self.assertEqual(current, expected)
        # delete from operation
        # the None argument in delete DML is included to avoid pylint E1120
        table.delete(None).where(table.c.column_int == 2).execute()
        expected = 1
        current = ora_conn.engine.scalar(
            select([func.count('*')]).select_from(table)
        )
        self.assertEqual(current, expected)
        ora_conn.drop('test_delete')

    def test_insert(self):
        """Check insert rows using SQL Loader."""
        ora_conn = Oracle(*self.conn_params)
        sql = f"""CREATE TABLE test_insert (ranking INTEGER,
            nickname VARCHAR(100), score NUMBER)"""
        ora_conn.execute(sql)
        data = [[10, 'tom', 80.5], [20, 'nick', 15.3], [30, 'juli', 'x']]
        df = pandas.DataFrame(data, columns=['ranking', 'nickname', 'score'])
        df.name = 'test_insert'
        ora_conn.insert(
            *self.conn_params,
            data_table=df,
            output_path=self.output_path,
            os_path='/opt/oracle/instantclient_21_4',
            os_ld_library_path='/opt/oracle/instantclient_21_4',
            mode='INSERT',
            schema=self.user,
            errors=1,
            columns=['*'],
            remove_data=False
        )
        table = ora_conn.get_table('test_insert')
        expected = 2
        current = ora_conn.engine.scalar(
            select([func.count('*')]).select_from(table)
        )
        self.assertEqual(current, expected)
        ora_conn.drop('test_insert')

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""Unit tests for etlstat."""

import os
import unittest

import etlstat.extractor.extractor as extractor

import pandas as pd


class TestExtractor(unittest.TestCase):
    """Unit tests for px."""

    def setUp(self):
        """Set initialization variables."""
        self.base_path = os.path.dirname(os.path.abspath(__file__)) + '/data'

    def test_csv(self):
        """Should massively read CSV files.

        Should read every csv file in a directory and generate a
        dict with csv names as keys and dataframes as values.

        """
        dir_path = self.base_path + '/csv/'
        data = extractor.csv(dir_path, sep=";")
        agenv_head = ['NORDEN', 'QI11', 'QI121', 'QI122', 'QI131', 'QI132',
                      'QI1331', 'QI1332', 'QI1333', 'QI1334', 'QI134', 'QI135',
                      'QI135E', 'QI14', 'QI15', 'QI15E', 'QI21', 'QI221',
                      'QI222', 'QI231', 'QI232', 'QI24', 'QI25', 'QJ11',
                      'QJ12', 'QJ131', 'QJ132', 'QJ133', 'QJ134', 'QJ14',
                      'QJ15', 'QJ16', 'QJ17', 'QJ18', 'QJ18E', 'QJ2', 'QJ3',
                      'QJ4', 'QJ5', 'QJ6', 'QJ6E']

        aereo_head = ['NORDEN', 'AI1', 'AI11', 'AI12', 'AI2', 'AI21', 'AI22',
                      'AI3', 'AI4', 'AI5', 'AJ1', 'AJ2', 'AJ3', 'AJ4', 'AJ5',
                      'AJ6', 'AJ7', 'AJ7E', 'AK111', 'AK121', 'AK131',
                      'AK112', 'AK122', 'AK132', 'AK211', 'AK221', 'AK231',
                      'AK212', 'AK222', 'AK232', 'AK213', 'AK223', 'AK233',
                      'AK214', 'AK224', 'AK234', 'AM1', 'AM11', 'AM2']

        aloj_head = ['NORDEN', 'HI11', 'HI12', 'HI13', 'HI14', 'HI15', 'HI16',
                     'HI17', 'HI18', 'HI19', 'HI19E', 'HI21', 'HI221', 'HI222',
                     'HI231', 'HI232', 'HI24', 'HI25', 'HI25E', 'HJ1', 'HJ2',
                     'HJ3', 'HJ4', 'HJ5', 'HJ6', 'HJ7', 'HJ8', 'HJ9', 'HJ10',
                     'HJ10E']

        self.assertEqual(data['AEREO_SER_06_15.csv'].shape[0], 2)
        self.assertEqual(data['AEREO_SER_06_15.csv'].shape[1], 39)
        self.assertEqual(data['AGENV_SER_06_15.csv'].shape[0], 7)
        self.assertEqual(data['AGENV_SER_06_15.csv'].shape[1], 41)
        self.assertEqual(data['ALOJ_SER_06_15.csv'].shape[0], 11)
        self.assertEqual(data['ALOJ_SER_06_15.csv'].shape[1], 30)
        self.assertEqual(list(data['AEREO_SER_06_15.csv'].columns),
                         aereo_head)
        self.assertEqual(list(data['AGENV_SER_06_15.csv'].columns),
                         agenv_head)
        self.assertEqual(list(data['ALOJ_SER_06_15.csv'].columns),
                         aloj_head)

    def test_txt(self):
        """Should massively read positional text files from a directory.

        Should read positional txt files in a directory with their formats
        in csv and return a dict.

        """
        dir_path = self.base_path + '/positional/'
        data = extractor.txt(dir_path, format_path=dir_path + 'format/')

        post_head = ['NORDEN', 'PI1', 'PI2', 'PI2R', 'PI2N', 'PI21', 'PI31',
                     'PI41', 'PI42', 'PJ1', 'PJ11', 'PJ12', 'PJ13', 'PJ2',
                     'PJ21', 'PJ22', 'PJ3', 'PJ4', 'PJ5', 'PK11', 'PK121',
                     'PK122', 'PK13', 'PK14', 'PK15', 'PK16', 'PK17', 'PK17E',
                     'PK21', 'PK22', 'PK23', 'PK31', 'PK32', 'PK33', 'PK34',
                     'PK35', 'PK36', 'PK37', 'PK38', 'PK39', 'PK310', 'PK311',
                     'PK312', 'PK313', 'PK314', 'PK315', 'PK316', 'PK316E',
                     'PL11', 'PL12', 'PL13', 'PL14', 'PL21', 'PL22', 'PL23',
                     'PL24', 'PL31', 'PL32', 'PL33', 'PL34', 'PL41', 'PL42',
                     'PL43', 'PL44', 'PL51', 'PL52', 'PL53', 'PL54', 'PL5E',
                     'PM1', 'PM21', 'PM22', 'PM3', 'PM4']

        post_type = ['O', 'float32', 'float32', 'O', 'O', 'float32', 'float32',
                     'float32', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'float32', 'float32', 'float32', 'float32',
                     'O', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'float32', 'float32', 'float32', 'float32',
                     'O', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'float32', 'float32', 'float32', 'float32',
                     'float32', 'O', 'float32', 'float32', 'float32',
                     'float32', 'float32']

        tec_head = ['NORDEN', 'KI111', 'KI112', 'KI113', 'KI114', 'KI115',
                    'KI116', 'KI121', 'KI122', 'KI123', 'KI124', 'KI125',
                    'KI126', 'KI127', 'KI128', 'KI129', 'I129E', 'KI131',
                    'KI132', 'KI133', 'KI134', 'KI134E', 'KI141', 'KI142',
                    'KI143', 'KI144', 'KI145', 'KI145E', 'KI15', 'KI16',
                    'KI16E', 'KI21', 'KI22', 'KI23', 'KI24', 'KI25', 'KI26',
                    'KI27', 'KI28', 'KI29', 'KI210', 'KI211', 'KI212',
                    'KI212E']

        tec_type = ['O', 'float32', 'float32', 'float32', 'float32', 'float32',
                    'float32', 'float32', 'float32', 'float32', 'float32',
                    'float32', 'float32', 'float32', 'float32', 'float32',
                    'O', 'float32', 'float32', 'float32', 'float32', 'O',
                    'float32', 'float32', 'float32', 'float32', 'float32', 'O',
                    'float32', 'float32', 'O', 'float32', 'float32', 'float32',
                    'float32', 'float32', 'float32', 'float32', 'float32',
                    'float32', 'float32', 'float32', 'float32', 'O']

        self.assertEqual(data['AEREO_SER_06_15.TXT'].shape[0], 2)
        self.assertEqual(data['AEREO_SER_06_15.TXT'].shape[1], 39)
        self.assertEqual(data['AGENV_SER_06_15.TXT'].shape[0], 7)
        self.assertEqual(data['AGENV_SER_06_15.TXT'].shape[1], 41)
        self.assertEqual(data['ALOJ_SER_06_15.TXT'].shape[0], 11)
        self.assertEqual(data['ALOJ_SER_06_15.TXT'].shape[1], 30)
        self.assertEqual(list(data['POST_SER_06_15.TXT'].columns), post_head)
        self.assertEqual(list(data['POST_SER_06_15.TXT'].dtypes), post_type)
        self.assertEqual(list(data['TEC_SER_06_15.TXT'].columns), tec_head)
        self.assertEqual(list(data['TEC_SER_06_15.TXT'].dtypes), tec_type)
        self.assertEqual(data['WHITE_SPACES.TXT']['COL_1'][1], ' a ')

    def test_xls(self):
        """Should massively read XLS files from a directory.

        Should read XLS files in a directory  and generate a dict with file and
        sheet names as keys and dataframes with the data as values.

        """
        dir_path = self.base_path + '/excel/'
        data = extractor.xls(dir_path, sep=None, encoding=None)

        self.assertEqual(len(data['prueba_excel.xls']), 4)
        self.assertEqual(len(data['excel_prueba.xls']), 4)
        self.assertEqual(type(data['prueba_excel.xls']['Hoja1']),
                         pd.core.frame.DataFrame)
        self.assertEqual(type(data['prueba_excel.xls']['Hoja3']),
                         pd.core.frame.DataFrame)
        self.assertEqual(type(data['excel_prueba.xls']['Hoja1']),
                         pd.core.frame.DataFrame)
        self.assertEqual(type(data['excel_prueba.xls']['Hoja2']),
                         pd.core.frame.DataFrame)

    def test_xlsx(self):
        """Should massively read XLSX files from a directory.

        Should read XLSX files in a directory  and generate a dict
        with file and sheet names as keys and dataframes with the data
        as values.

        """
        dir_path = self.base_path + '/excel/'
        data = extractor.xlsx(dir_path, sep=None, encoding=None)

        self.assertEqual(len(data['prueba_excel.xlsx']), 4)
        self.assertEqual(len(data['excel_prueba.xlsx']), 4)
        self.assertEqual(type(data['prueba_excel.xlsx']['Hoja1']),
                         pd.core.frame.DataFrame)
        self.assertEqual(type(data['prueba_excel.xlsx']['Hoja3']),
                         pd.core.frame.DataFrame)
        self.assertEqual(type(data['excel_prueba.xlsx']['Hoja1']),
                         pd.core.frame.DataFrame)
        self.assertEqual(type(data['excel_prueba.xlsx']['Hoja2']),
                         pd.core.frame.DataFrame)

    def test_xml(self):
        """Should massively read XML files from a directory.

        Should massively read XML files in a directory and generate a dict with
        the file names as keys and the etree objects as values.

        """
        xml_dict = extractor.xml(self.base_path + '/xml/')
        self.assertEqual(len(xml_dict), 11)
        root = xml_dict['Comex.kjb'].getroot()
        self.assertEqual(root.tag, 'job')
        root = xml_dict['Ec_SE_IEFAZ.ktr'].getroot()
        self.assertEqual(root.tag, 'transformation')

    def test_sql(self):
        """Should massively read SQL files from a directory.

        Should read every SQL file in a given directory and generate a dict
        with the file names as keys and the content of the files as values.

        """
        sql_data = extractor.sql(self.base_path + '/sql/')
        self.assertEqual(len(sql_data['afiliados']), 1582)
        self.assertEqual(len(sql_data['contratos']), 3023)

    def test_px(self):
        """Should massively read PC-Axis files from a list of URLs in a CSV.

        Should read every px file from the URIs listed in an input CSV file and
        generate a dict with the px name as key and a dataframe with its data
        as value.

        """
        line1 = 'id,url\n'
        line2 = 'px_o20012,' + self.base_path + '/px/o20012.px\n'
        line3 = 'px_o20013,' + self.base_path + '/px/o20013.px'
        file = open(self.base_path + '/px/pcaxis_urls.csv', "w")
        file.write(line1)
        file.write(line2)
        file.write(line3)
        file.close()
        pc_axis_data = extractor.px(self.base_path + '/px/pcaxis_urls.csv')
        self.assertEqual(len(pc_axis_data), 2)
        self.assertEqual(type(pc_axis_data['px_o20012']),
                         pd.core.frame.DataFrame)
        self.assertEqual(
            type(pc_axis_data['px_o20013']), pd.core.frame.DataFrame)

    def test_px_file(self):
        """Should massively read PC-Axis files in a directory.

        Should read every px file from a given path (directory).

        """
        pc_axis_data = extractor.px(self.base_path + '/px_file/')
        self.assertEqual(len(pc_axis_data), 1)
        self.assertEqual(len(pc_axis_data['27066']), 152064)
        self.assertEqual(pc_axis_data['27066']['DATA'][29], '98.728')
        self.assertEqual(
            type(pc_axis_data['27066']), pd.core.frame.DataFrame)

    def test_html_table(self):
        """Should massively read html files in a directory.

        Should read every html file and generate a dict
        with the name as key and the content as value.
        """
        html_table_data = extractor.html_table(
            self.base_path + '/html_table/', encoding='utf-8')
        self.assertEqual(type(html_table_data), dict)
        self.assertEqual(len(html_table_data), 2)

    def test_html_table_file(self):
        """Should read an html file with a table, and loads the table."""
        without_thead_df = extractor._html_table_file(
            self.base_path + '/html_table/test.html', encoding='utf-8')
        self.assertEqual(type(without_thead_df), pd.DataFrame)
        self.assertEqual(without_thead_df['Savings'][1], '$200')
        self.assertEqual(without_thead_df.shape, (2, 2))

        with_thead_df = extractor._html_table_file(
            self.base_path + '/html_table/icane_economy.html', encoding='utf-8')
        self.assertEqual(type(with_thead_df), pd.DataFrame)
        self.assertEqual(with_thead_df['Cantabria_Dato'][0], '96,9')
        self.assertEqual(with_thead_df.shape, (15, 6))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""Integration tests for MySQL database module."""

import os
import unittest
import pandas as pd

from etlstat.database.mysql import MySQL
from etlstat.text import utils

from pyaxis import pyaxis

from sqlalchemy import (Boolean, Column, DateTime, Float, Integer, String,
                        func, select)
from sqlalchemy.exc import InvalidRequestError
from sqlalchemy.ext.declarative import declarative_base


class TestMySQL(unittest.TestCase):
    """Testing methods for MySQL class."""

    user = 'test'
    password = 'password'
    host = '127.0.0.1'
    port = '3306'
    database = 'test'
    conn_params = [user, password, host, port, database]

    @classmethod
    def setUpClass(cls):
        """Set up test variables."""
        user = 'root'
        password = 'password'
        host = '127.0.0.1'
        port = '3306'
        database = ''
        conn_params = [user, password, host, port, database]
        ddl = "DROP DATABASE IF EXISTS test"
        my_conn = MySQL(*conn_params)
        my_conn.execute(ddl)
        ddl = "CREATE DATABASE test"
        my_conn.execute(ddl)
        sql = "CREATE TABLE IF NOT EXISTS test.inf_schema as " \
              "SELECT table_name, avg_row_length, " \
              "table_collation, create_time " \
              "FROM information_schema.tables"
        my_conn.execute(sql)
        ddl = "GRANT ALL PRIVILEGES ON test.* to 'test'@'%'" \
              " IDENTIFIED BY 'password'"
        ddl_file = "GRANT FILE ON *.* to 'test'@'%' IDENTIFIED BY 'password'"
        my_conn.execute(ddl)
        my_conn.execute(ddl_file)

    def test_init(self):
        """Check connection with MySQL database."""
        self.assertEqual(str(MySQL(*self.conn_params).engine),
                         "Engine(mysql+mysqlconnector://test:***@127.0.0.1:"
                         "3306/test)")

    def test_get_table(self):
        """Check get table from the database using SqlAlchemy."""
        my_conn = MySQL(*self.conn_params)
        inf_schema = my_conn.get_table('inf_schema')  # GET TABLE example
        row_count = my_conn.engine.scalar(
            select([func.count('*')]).select_from(inf_schema)
        )
        # The select.columns parameter is not available in the method form of
        # select(), e.g. FromClause.select().
        # See https://docs.sqlalchemy.org/en/latest/core/selectable.html#
        # sqlalchemy.sql.expression.FromClause.select
        my_conn.engine.execute(
            select([inf_schema.c.table_name]).select_from(inf_schema))
        self.assertGreaterEqual(row_count, 100)

    def test_execute(self):
        """Check execute method launching arbitrary sql queries."""
        my_conn = MySQL(*self.conn_params)
        sql = f'''CREATE TABLE table1 (id integer, column1 varchar(100),
                  column2 double)'''
        my_conn.execute(sql)
        table1 = my_conn.get_table('table1')
        self.assertEqual(table1.c.column1.name, 'column1')
        sql = "INSERT INTO table1 (id, column1, column2) " \
              "VALUES (1, 'Varchar text (100 char)', 123456789.0123456789)"
        my_conn.execute(sql)  # EXECUTE example
        # The select.columns parameter is not available in the method form of
        # select(), e.g. FromClause.select().
        # See https://docs.sqlalchemy.org/en/latest/core/selectable.html#
        # sqlalchemy.sql.expression.FromClause.select
        results = my_conn.engine.execute(
            select([table1.c.column1]).select_from(table1)).fetchall()
        expected = 'Varchar text (100 char)'
        current = results[0][0]
        # this returns a tuple inside a list and I dont know why
        self.assertEqual(expected, current)
        query = 'select * from table1 order by id'
        result = my_conn.execute(query)
        expected = 1
        current = len(result[0].index)
        self.assertEqual(expected, current)
        my_conn.drop('table1')

    def test_execute_multiple(self):
        """Check if multiple SQL statements are correctly executed."""
        my_conn = MySQL(*self.conn_params)
        sql = f"""CREATE TABLE table1 (id integer, column1 varchar(100),
              column2 double);
              INSERT INTO table1 (id, column1, column2)
              VALUES (1, 'Varchar; text; (100 char)',
              123456789.012787648484859);
              INSERT INTO table1 (id, column1, column2)
              VALUES (2, 'Varchar; text; (100 char)',
              -789.0127876);
              SELECT id, column2 FROM table1;"""
        results = my_conn.execute(sql)
        self.assertEqual(len(results), 4)
        self.assertEqual(len(results[3].index), 2)
        my_conn.drop('table1')

    def test_drop(self):
        """Check drop for an existing table."""
        my_conn = MySQL(*self.conn_params)
        sql = "CREATE TABLE table1 (id integer, column1 varchar(100), " \
            "column2 double)"
        my_conn.execute(sql)
        my_conn.get_table('table1')
        my_conn.drop('table1')  # DROP example
        with self.assertRaises(InvalidRequestError):
            my_conn.get_table('table1')

    def test_create(self):
        """Check create table using sqlalchemy."""
        Base = declarative_base()
        my_conn = MySQL(*self.conn_params)

        # table creation can be done via execute() + raw SQL or using this:
        class Table2(Base):
            """Auxiliary sqlalchemy table model for the tests."""

            __tablename__ = 'table2'

            column_int = Column(Integer)
            column_string = Column(String(20))
            column_float = Column(Float)
            column_datetime = Column(DateTime)
            column_boolean = Column(Boolean)
            id = Column(Integer, primary_key=True)

        Table2.__table__.create(bind=my_conn.engine)
        table2 = my_conn.get_table('table2')
        self.assertEqual(table2.c.column_datetime.name, 'column_datetime')
        self.assertEqual(len(table2.c), 6)
        my_conn.drop('table2')

    def test_select(self):
        """Check select statement using sqlalchemy."""
        my_conn = MySQL(*self.conn_params)
        table_name = "inf_schema"
        inf_schema = my_conn.get_table(table_name)
        # SELECT * FROM inf_schema
        # WHERE table_name like 'INNO%' AND avg_row_length > 100
        results = my_conn.engine.execute(select('*')
                                         .where(inf_schema.c.table_name
                                                .like('INNO%'))
                                         .where(inf_schema.c.avg_row_length >
                                                100)
                                         .select_from(inf_schema)).fetchall()
        table_df = pd.DataFrame(results)
        self.assertGreaterEqual(len(table_df), 6)

    def test_insert(self):
        """Check that insert method inserts rows into a table."""
        my_conn = MySQL(*self.conn_params)
        Base = declarative_base()
        current_dir = os.path.dirname(os.path.abspath(__file__))

        class Pmh(Base):
            """Auxiliary sqlalchemy table model for the tests."""

            __tablename__ = 'pmh'

            id = Column(Integer, primary_key=True)
            anno = Column(Integer)
            cpro = Column(Integer)
            cmun = Column(Integer)
            csexo = Column(Integer)
            sexo = Column(String(20))
            orden_gredad = Column(Integer)
            gredad = Column(String(30))
            personas = Column(Integer)
            codigo_ine = Column(String(50))

        Pmh.__table__.create(bind=my_conn.engine)
        data = pd.read_csv(f'''{current_dir}/pmh.csv''')
        data.name = 'pmh'
        my_conn.insert(data, if_exists='append')
        # my_conn.execute(f'''alter table {data.name} add primary key(id)''')
        actual = my_conn.engine.scalar(
            select([func.count('*')]).select_from(Pmh)
        )
        expected = len(data.index)
        self.assertEqual(actual, expected)
        my_conn.drop('pmh')

    def test_upsert(self):
        """Check that upsert method inserts or updates rows in a table."""
        my_conn = MySQL(*self.conn_params)
        Base = declarative_base()
        current_dir = os.path.dirname(os.path.abspath(__file__))

        # IMPORTANT: table logic reuse pattern with mixins
        class PmhMixin(object):
            """Auxiliary sqlalchemy table model for the tests."""

            __tablename__ = 'pmh'

            id = Column(Integer, primary_key=True)
            anno = Column(Integer)
            cpro = Column(Integer)
            cmun = Column(Integer)
            csexo = Column(Integer)
            sexo = Column(String(20))
            orden_gredad = Column(Integer)
            gredad = Column(String(30))
            personas = Column(Integer)
            codigo_ine = Column(String(50))

        class Pmh(Base, PmhMixin):
            """Auxiliary sqlalchemy table model for the tests."""

            __tablename__ = 'pmh'

        class PmhTmp(Base, PmhMixin):
            """Auxiliary sqlalchemy table model for the tests."""

            __tablename__ = 'tmp_pmh'

        # table to update/insert
        Pmh.__table__.create(bind=my_conn.engine)
        data = pd.read_csv(f'''{current_dir}/pmh.csv''')
        data.name = 'pmh'
        my_conn.insert(data, if_exists='append')
        # https://github.com/PyCQA/pylint/issues/1161
        # there's an issue with pylint and pandas read methods.
        original_table = pd.DataFrame(
            pd.read_sql_table(data.name, my_conn.conn_string))

        # temporary table with data to update/insert
        PmhTmp.__table__.create(bind=my_conn.engine)
        tmp_data = pd.read_csv(f'''{current_dir}/pmh_update.csv''')
        tmp_data.name = 'tmp_pmh'

        sql = f'''INSERT INTO pmh
                   (id, anno, cpro, cmun, csexo, sexo, orden_gredad, gredad,
                  personas, codigo_ine)
                  SELECT *
                  FROM tmp_pmh
                  ON DUPLICATE KEY UPDATE
                  id = tmp_pmh.id,
                  anno = tmp_pmh.anno,
                  cpro = tmp_pmh.cpro,
                  cmun = tmp_pmh.cmun,
                  csexo = tmp_pmh.csexo,
                  orden_gredad = tmp_pmh.orden_gredad,
                  gredad = tmp_pmh.gredad,
                  personas = tmp_pmh.personas,
                  codigo_ine = tmp_pmh.codigo_ine;'''
        expected = 76
        current = original_table.loc[
            original_table['id'] == 5192]['personas'].tolist()[0]
        self.assertEqual(current, expected)
        my_conn.upsert(tmp_data, data.name, sql, if_exists='append')
        updated_table = pd.DataFrame(
            pd.read_sql_table(data.name, my_conn.conn_string))
        expected = 9976
        current = updated_table.loc[
            updated_table['id'] == 5192]['personas'].tolist()[0]
        self.assertEqual(current, expected)
        expected = 46
        current = updated_table.loc[
            updated_table['id'] == 30001]['cmun'].tolist()[0]
        my_conn.drop(data.name)

    def test_delete(self):
        """Check delete rows from table."""
        data_columns = ['id', 'column_string', 'column_float']
        data_values = [[1, 'string1', 456.956], [2, 'string2', 38.905]]
        data = pd.DataFrame(data_values, columns=data_columns)
        data.name = 'test_delete'
        my_conn = MySQL(*self.conn_params)
        my_conn.insert(data)
        table = my_conn.get_table(data.name)
        expected = 2
        current = my_conn.engine.scalar(
            select([func.count('*')]).select_from(table)
        )
        self.assertEqual(current, expected)

        # delete from operation
        # the None argument in delete DML is included to avoid pylint E1120
        table.delete(None).where(table.c.id == 2).execute()

        expected = 1
        current = my_conn.engine.scalar(
            select([func.count('*')]).select_from(table)
        )
        self.assertEqual(current, expected)
        my_conn.drop(data.name)

    def test_complete_insert_from_pcaxis(self):
        """Check complete cycle by inserting a pcaxis file into a table."""
        my_conn = MySQL(*self.conn_params)
        Base = declarative_base()
        current_dir = os.path.dirname(os.path.abspath(__file__))
        parsed_pcaxis = pyaxis.parse(current_dir + '/22350.px',
                                     encoding='ISO-8859-1')
        table_data = parsed_pcaxis['DATA']
        table_data = utils.parse_df_columns(table_data)
        table_data.name = 'ipc'

        class Ipc(Base):
            """Auxiliary sqlalchemy table model for the tests."""

            __tablename__ = 'ipc'

            id = Column(Integer, primary_key=True)
            comunidades_y_ciudades_autonomas = Column(String(100))
            grupos_ecoicop = Column(String(100))
            tipo_de_dato = Column(String(100))
            periodo = Column(String(50))
            data = Column(Float)

        Ipc.__table__.create(bind=my_conn.engine)
        my_conn.insert(table_data, if_exists='append')
        actual = my_conn.engine.scalar(
            select([func.count('*')]).select_from(Ipc)
        )
        expected = len(table_data.index)
        self.assertEqual(actual, expected)
        my_conn.drop('ipc')

    def test_insert_selected_columns(self):
        """Check insert method only with selected columns."""
        my_conn = MySQL(*self.conn_params)
        Base = declarative_base()
        current_dir = os.path.dirname(os.path.abspath(__file__))
        parsed_pcaxis = pyaxis.parse(current_dir + '/22350.px',
                                     encoding='ISO-8859-1')
        table_data = parsed_pcaxis['DATA']
        table_data = utils.parse_df_columns(table_data)
        table_data.name = 'ipc'

        class Ipc(Base):
            """Auxiliary sqlalchemy table model for the tests."""

            __tablename__ = 'ipc'

            id = Column(Integer, primary_key=True)
            comunidades_y_ciudades_autonomas = Column(String(100))
            grupos_ecoicop = Column(String(100))
            tipo_de_dato = Column(String(50))
            periodo = Column(String(50))
            data = Column(Float)

        Ipc.__table__.create(bind=my_conn.engine)
        insert_data = table_data[['grupos_ecoicop', 'data']]
        insert_data.name = 'ipc'

        my_conn.insert(insert_data, if_exists='append',
                       columns=['grupos_ecoicop', 'data'])
        result_data = pd.read_sql_query('select * from ipc',
                                        con=my_conn.engine)
        self.assertTrue(
            result_data['comunidades_y_ciudades_autonomas'].isnull().all())
        self.assertTrue(result_data['periodo'].isnull().all())
        self.assertTrue(result_data['tipo_de_dato'].isnull().all())
        self.assertFalse(result_data['grupos_ecoicop'].isnull().all())
        self.assertFalse(result_data['data'].isnull().all())
        my_conn.drop('ipc')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from setuptools import find_packages, setup


setup(
    name='etlstat',
    version='0.9.3',
    author='Instituto Cántabro de Estadística',
    author_email='icane@cantabria.es',
    packages=find_packages(),
    url='https://github.com/icane/etlstat.git',
    license='Apache License 2.0',
    description='ETL utils for statistical offices data processing',
    long_description=open('README.rst').read(),
    install_requires=[
        'beautifulsoup4==4.*',
        'defusedxml==0.6.*',
        'pandas==1.4.*',
        'pyaxis==0.3.*',
        'python_Levenshtein==0.20.*',
        'SQLAlchemy==1.4.*',
        'sqlparse==0.3.*',
        'Unidecode==1.1.*',
        'cx_Oracle==7.3.*',
        'mysql-connector==2.2.*',
        'xlrd==2.0.*',
        'openpyxl==3.0.*'
    ],
    test_suite='extractor.test, database.test, text.test',
    keywords=['etl', 'icane', 'statistics', 'utils'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Software Development :: Libraries'
    ],
)



"""Integration tests for PostgreSQL database module."""

import unittest

from etlstat.database.postgresql import PostgreSQL


class TestPostgreSQL(unittest.TestCase):
    """Testing methods for Postgresql class."""

    user = 'test'
    password = 'password'
    host = '127.0.0.1'
    port = '5432'
    database = 'postgres'
    conn_params = [user, password, host, port, database]

    @classmethod
    def setUpClass(cls):
        """Set up test variables."""
        user = 'postgres'
        password = 'password'
        host = '127.0.0.1'
        port = '5432'
        database = 'postgres'
        conn_params = [user, password, host, port, database]
        ddl = "DROP SCHEMA IF EXISTS test CASCADE"
        pg_conn = PostgreSQL(*conn_params)
        pg_conn.execute(ddl)
        ddl = "DROP USER IF EXISTS test"
        pg_conn.execute(ddl)
        ddl = "CREATE SCHEMA test"
        pg_conn.execute(ddl)
        sql = "CREATE TABLE IF NOT EXISTS test.inf_schema AS " \
              "SELECT table_catalog, table_schema, " \
              "table_name, table_type " \
              "FROM information_schema.tables"
        pg_conn.execute(sql)
        ddl = "CREATE USER test WITH ENCRYPTED PASSWORD 'password'"
        pg_conn.execute(ddl)
        ddl = "GRANT ALL PRIVILEGES ON SCHEMA test TO test"
        pg_conn.execute(ddl)

    def test_init(self):
        """Check connection with Postgresql database."""
        self.assertEqual(str(PostgreSQL(*self.conn_params).engine),
                         "Engine(postgresql://test:***@127.0.0.1:"
                         "5432/postgres)")

    def test_execute(self):
        """Check execute method launching arbitrary sql queries."""
        pg_conn = PostgreSQL(*self.conn_params)
        sql = f'''CREATE TABLE table1 (id integer, column1 varchar(100),
                  column2 float)'''
        pg_conn.execute(sql)
        sql = "INSERT INTO table1 (id, column1, column2) " \
              "VALUES (1, 'Varchar text (100 char)', 123456789.0123456789)"
        pg_conn.execute(sql)
        result = pg_conn.execute("SELECT * FROM table1")
        self.assertEqual('Varchar text (100 char)', result[0]['column1'][0])
        query = 'SELECT * FROM table1 ORDER BY id'
        result = pg_conn.execute(query)
        expected = 1
        current = len(result[0].index)
        self.assertEqual(expected, current)
        pg_conn.execute('DROP TABLE table1')

    def test_execute_multiple(self):
        """Check if multiple SQL statements are correctly executed."""
        pg_conn = PostgreSQL(*self.conn_params)
        sql = f"""CREATE TABLE table1 (id integer, column1 varchar(100),
              column2 float);
              INSERT INTO table1 (id, column1, column2)
              VALUES (1, 'Varchar; text; (100 char)',
              123456789.012787648484859);
              INSERT INTO table1 (id, column1, column2)
              VALUES (2, 'Varchar; text; (100 char)',
              -789.0127876);
              SELECT id, column2 FROM table1;"""
        result = pg_conn.execute(sql)
        self.assertEqual(len(result), 4)
        self.assertEqual(len(result[3].index), 2)
        pg_conn.execute('DROP TABLE table1')


if __name__ == '__main__':
    unittest.main()
